/*
 *    Copyright 2009-2010 The iBaGuice Team
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.googlecode.ibaguice.cache.jbosscache;

import java.io.IOException;
import java.io.InputStream;
import java.util.Set;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import org.jboss.cache.Cache;
import org.jboss.cache.CacheFactory;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;

/**
 * Cache adapter for JBoss Cache.
 *
 * @version $Id: JBossCache.java 2095 2010-06-04 13:56:24Z simone.tripodi $
 */
public final class JBossCache implements org.apache.ibatis.cache.Cache {

    private static final Cache<Object, Object> CACHE = createCache();

    private static final String FQN_FORMAT = "/ibatis/%s";

    /**
     * Looks for "/jboss-cache.xml" classpath resource and builds the relative
     * {@code Cache}; if it's no found or it is impossible to load it,
     * returns the default Cache.
     *
     * @return the application cache manager.
     */
    private static Cache<Object, Object> createCache() {
        CacheFactory<Object, Object> factory = new DefaultCacheFactory<Object, Object>();
        Cache<Object, Object> cache;

        InputStream input = JBossCache.class.getResourceAsStream("/jboss-cache.xml");

        if (input != null) {
            try {
                cache = factory.createCache(input);
            } catch (Throwable t) {
                cache = factory.createCache();
            } finally {
                try {
                    input.close();
                } catch (IOException e) {
                }
            }
        } else {
            cache = factory.createCache();
        }

        return cache;
    }

    /**
     * The {@link ReadWriteLock}.
     */
    private final ReadWriteLock readWriteLock = new ReentrantReadWriteLock();

    /**
     * The cache id.
     */
    private final String id;

    private final Fqn<String> fqn;

    /**
     * Instantiates a new JBoss cache for the specified id.
     *
     * @param id the cache id.
     */
    public JBossCache(final String id) {
        this.id = id;
        this.fqn = Fqn.fromString(String.format(FQN_FORMAT, this.id));
    }

    /**
     * {@inheritDoc}
     */
    public void clear() {
        CACHE.clearData(this.fqn);
    }

    /**
     * {@inheritDoc}
     */
    public String getId() {
        return this.id;
    }

    /**
     * {@inheritDoc}
     */
    public Object getObject(Object key) {
        return CACHE.get(this.fqn, key);
    }

    /**
     * {@inheritDoc}
     */
    public ReadWriteLock getReadWriteLock() {
        return this.readWriteLock;
    }

    /**
     * {@inheritDoc}
     */
    public int getSize() {
        Set<Object> keys = CACHE.getKeys(this.fqn);
        if (keys == null) {
            return 0;
        }
        return keys.size();
    }

    /**
     * {@inheritDoc}
     */
    public void putObject(Object key, Object value) {
        CACHE.put(this.fqn, key, value);
    }

    /**
     * {@inheritDoc}
     */
    public Object removeObject(Object key) {
        Object ret = this.getObject(key);
        if (ret != null) {
            CACHE.remove(this.fqn, key);
        }
        return ret;
    }

}
